 /* Arduino UNO rev 3
 *  I/O list:
 *  - OLED SSD1306
 *  - DHT11
 *  - MH-Z14 (SEN0219)
 *  
 *  Connections:
 *  - Arduino.D2  - DHT11.out
 *  - Arduino.D3  - MH-Z14.6 (PWM output)
 *  - Arduino.SCL - OLED.SLK
 *  - Arduino.SDA - OLED.SDA
 */
#include <Wire.h>
#include <SPI.h>
// two libraries inserted into the 'libraries' folder
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
// dht library
#include "DHT.h"

#define DHTPIN 2     // what digital pin we're connected to
#define OLED_RESET 4
Adafruit_SSD1306 display(OLED_RESET);

#define CO2_SENSOR_PWM_PIN 3
unsigned long duration;
long ppm;
#include <SPI.h>

#define DHTTYPE DHT11 

#if (SSD1306_LCDHEIGHT != 64)
#error("Height incorrect, please fix Adafruit_SSD1306.h!");
#endif

DHT dht(DHTPIN, DHTTYPE);

void setup()   {                
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);  // initialize with the I2C addr 0x3D (for the 128x64)
  display.display(); // show splashscreen
  delay(2000);
  Serial.begin(9600);
  pinMode(CO2_SENSOR_PWM_PIN, INPUT);
  Serial.println("CO2_ppm program start.");
  Serial.println();
  display.clearDisplay();   // clears the screen and buffer
  dht.begin();
  delay(2000);
}


void loop() {
  float h = dht.readHumidity();
  // Read temperature as Celsius (the default)
  float t = dht.readTemperature();
  // Read temperature as Fahrenheit (isFahrenheit = true)
  float f = dht.readTemperature(true);

  // Check if any reads failed.
  if (isnan(h) || isnan(t) || isnan(f)) {
    delay(2000);
  } else{
    // routine for converting temp/hum floats to char arrays
    char temp_buff[5]; char hum_buff[5];
    char temp_disp_buff[11] = "Tmp:";
    char hum_disp_buff[11] = "Hum:";
    
    // appending temp/hum to buffers
    dtostrf(t,2,1,temp_buff);
    strcat(temp_disp_buff,temp_buff);
    dtostrf(h,2,1,hum_buff);
    strcat(hum_disp_buff,hum_buff);

     while(digitalRead(CO2_SENSOR_PWM_PIN) == HIGH) {;}
    //wait for the pin to go HIGH and measure HIGH time
    duration = pulseIn(CO2_SENSOR_PWM_PIN, HIGH, 2000000);
    ppm = 5000 * ((duration/1000) - 2)/1000;
    //Serial.print(ppm);
    Serial.print(ppm);
    Serial.println();
    
    // routine for displaying text for temp/hum readout
    display.clearDisplay();
    display.setTextSize(2);
    display.setTextColor(WHITE);
    display.setCursor(0,0);
    display.println(temp_disp_buff);
    display.println(hum_disp_buff);
    display.println("Co2:");
    Serial.print(ppm);
    display.setCursor(48,32);
    display.println(ppm);
    display.display();
    delay(2000);
  }
}
